# Source Generated with Decompyle++
# File: in.pyo (Python 2.4)

'''A multi-producer, multi-consumer queue.'''
from time import time as _time
from collections import deque
__all__ = [
    'Empty',
    'Full',
    'Queue']

class Empty(Exception):
    '''Exception raised by Queue.get(block=0)/get_nowait().'''
    pass


class Full(Exception):
    '''Exception raised by Queue.put(block=0)/put_nowait().'''
    pass


class Queue:
    
    def __init__(self, maxsize = 0):
        '''Initialize a queue object with a given maximum size.

        If maxsize is <= 0, the queue size is infinite.
        '''
        
        try:
            import threading as threading
        except ImportError:
            import dummy_threading as threading

        self._init(maxsize)
        self.mutex = threading.Lock()
        self.not_empty = threading.Condition(self.mutex)
        self.not_full = threading.Condition(self.mutex)

    
    def qsize(self):
        '''Return the approximate size of the queue (not reliable!).'''
        self.mutex.acquire()
        n = self._qsize()
        self.mutex.release()
        return n

    
    def empty(self):
        '''Return True if the queue is empty, False otherwise (not reliable!).'''
        self.mutex.acquire()
        n = self._empty()
        self.mutex.release()
        return n

    
    def full(self):
        '''Return True if the queue is full, False otherwise (not reliable!).'''
        self.mutex.acquire()
        n = self._full()
        self.mutex.release()
        return n

    
    def put(self, item, block = True, timeout = None):
        """Put an item into the queue.

        If optional args 'block' is true and 'timeout' is None (the default),
        block if necessary until a free slot is available. If 'timeout' is
        a positive number, it blocks at most 'timeout' seconds and raises
        the Full exception if no free slot was available within that time.
        Otherwise ('block' is false), put an item on the queue if a free slot
        is immediately available, else raise the Full exception ('timeout'
        is ignored in that case).
        """
        self.not_full.acquire()
        
        try:
            if not block:
                if self._full():
                    raise Full
                
            elif timeout is None:
                while self._full():
                    self.not_full.wait()
            elif timeout < 0:
                raise ValueError("'timeout' must be a positive number")
            
            endtime = _time() + timeout
            while self._full():
                remaining = endtime - _time()
                if remaining <= 0.0:
                    raise Full
                
                self.not_full.wait(remaining)
            self._put(item)
            self.not_empty.notify()
        finally:
            self.not_full.release()


    
    def put_nowait(self, item):
        '''Put an item into the queue without blocking.

        Only enqueue the item if a free slot is immediately available.
        Otherwise raise the Full exception.
        '''
        return self.put(item, False)

    
    def get(self, block = True, timeout = None):
        """Remove and return an item from the queue.

        If optional args 'block' is true and 'timeout' is None (the default),
        block if necessary until an item is available. If 'timeout' is
        a positive number, it blocks at most 'timeout' seconds and raises
        the Empty exception if no item was available within that time.
        Otherwise ('block' is false), return an item if one is immediately
        available, else raise the Empty exception ('timeout' is ignored
        in that case).
        """
        self.not_empty.acquire()
        
        try:
            if not block:
                if self._empty():
                    raise Empty
                
            elif timeout is None:
                while self._empty():
                    self.not_empty.wait()
            elif timeout < 0:
                raise ValueError("'timeout' must be a positive number")
            
            endtime = _time() + timeout
            while self._empty():
                remaining = endtime - _time()
                if remaining <= 0.0:
                    raise Empty
                
                self.not_empty.wait(remaining)
            item = self._get()
            self.not_full.notify()
            return item
        finally:
            self.not_empty.release()


    
    def get_nowait(self):
        '''Remove and return an item from the queue without blocking.

        Only get an item if one is immediately available. Otherwise
        raise the Empty exception.
        '''
        return self.get(False)

    
    def _init(self, maxsize):
        self.maxsize = maxsize
        self.queue = deque()

    
    def _qsize(self):
        return len(self.queue)

    
    def _empty(self):
        return not (self.queue)

    
    def _full(self):
        if self.maxsize > 0:
            pass
        return len(self.queue) == self.maxsize

    
    def _put(self, item):
        self.queue.append(item)

    
    def _get(self):
        return self.queue.popleft()


